<?php

namespace Drupal\view_mode_selector_field\Plugin\Field\FieldWidget;

use Drupal\Core\Extension\Extension;
use Drupal\Core\Extension\ThemeHandlerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines the 'view_mode_selector_field_widget' field widget.
 *
 * @FieldWidget(
 *   id = "view_mode_selector_field_widget",
 *   label = @Translation("View Mode Selector Field Widget"),
 *   field_types = {"view_mode_selector_field_type"},
 * )
 */
class ViewModeSelectorFieldWidget extends WidgetBase {

  /**
   * Field directory.
   *
   * @const string
   */
  const FIELD_DIR = 'dir';

  /**
   * Theme.
   *
   * @const string
   */
  const FIELD_THEME = 'theme';

  /**
   * Group.
   *
   * @const string
   */
  const FIELD_GROUP = 'group';

  /**
   * Theme Handler.
   *
   * @var \Drupal\Core\Extension\ThemeHandlerInterface
   */
  protected ThemeHandlerInterface $themeHandler;

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected FileSystemInterface $fileSystem;

  /**
   * Drupal root.
   *
   * @var string
   */
  protected $root;

  /**
   * {@inheritdoc}
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, ThemeHandlerInterface $themeHandler, FileSystemInterface $fileSystem, string $root) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);

    $this->themeHandler = $themeHandler;
    $this->fileSystem = $fileSystem;
    $this->root = $root;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('theme_handler'),
      $container->get('file_system'),
      $container->getParameter('app.root')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      static::FIELD_DIR   => 'view_mode_thumbnails',
      static::FIELD_THEME => static::getDefaultActiveTheme(),
      static::FIELD_GROUP => 'default',
    ] + parent::defaultSettings();
  }

  /**
   * Return the current active theme.
   */
  public static function getDefaultActiveTheme() {
    $themeSystem = \Drupal::config('system.theme');

    return $themeSystem->get('default');
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = [];

    $element[static::FIELD_DIR] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Thumbnails directory (in the front theme)'),
      '#default_value' => $this->getSetting(static::FIELD_DIR),
      '#required'      => TRUE,
    ];

    $element[static::FIELD_THEME] = [
      '#type'          => 'select',
      '#options'       => $this->getThemeList(),
      '#title'         => $this->t('The theme where thumbnails are stocked'),
      '#default_value' => $this->getSetting(static::FIELD_THEME),
      '#required'      => TRUE,
    ];

    $element[static::FIELD_GROUP] = [
      '#type'          => 'select',
      '#options'       => $this->getGroupList($form),
      '#title'         => $this->t('The group where thumbnails are stocked'),
      '#default_value' => $this->getSetting(static::FIELD_GROUP),
      '#required'      => TRUE,
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary[] = $this->t('Theme : @theme, Dir: @dir, Group: @grp', [
      '@theme' => $this->getSetting(static::FIELD_THEME),
      '@dir'   => $this->getSetting(static::FIELD_DIR),
      '@grp'   => $this->getSetting(static::FIELD_GROUP),
    ]);

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    // Get the dir path according to settings.
    $dirPath = $this->getDirPath($items);

    if ($dirPath) {
      // Get Image path from dir path.
      $imgPaths = $this->getImgPath($dirPath);

      // Build the form.
      if (count($imgPaths) > 1) {
        $defaultValue = isset($items->getValue()[$delta]['value']) ? $items->getValue()[$delta]['value'] : NULL;
        $element = $this->getFormElement($element, $imgPaths, $defaultValue);
      }
    }
    else {
      $element = [];
    }

    return $element;
  }

  /**
   * Return the list of available themes.
   *
   * @return array|string[]
   *   THe list of available themes name.
   */
  protected function getThemeList() {
    return array_map(function (Extension $extension) {
      return $extension->getName();
    }, $this->themeHandler->listInfo());
  }

  /**
   * Return the list of folders which on the folder of the bundle.
   *
   * @param array $form
   *   The form.
   *
   * @return array|false
   *   The list of available groupes name.
   */
  protected function getGroupList(array $form) {
    $dir = implode('/', [
      $this->root,
      $this->themeHandler->getTheme($this->getSetting(static::FIELD_THEME))
        ->getPath(),
      $this->getSetting(static::FIELD_DIR),
      $form['#entity_type'],
      $form['#bundle'],
    ]);

    $groupDir = $dir . '/' . $this->getSetting(static::FIELD_GROUP);
    if (!is_dir($groupDir)) {
      $this->fileSystem->prepareDirectory($groupDir, FileSystemInterface::MODIFY_PERMISSIONS | FileSystemInterface::CREATE_DIRECTORY);
    }

    $scan = scandir($dir);
    $folders = array_filter($scan, function (string $folder) {
      return ($folder != '..' && $folder != '.');
    });

    return array_combine($folders, $folders);
  }

  /**
   * Return the dir path according to settings.
   *
   * @return string|bool
   *   THe dir path or false if the dirpath does not exist.
   */
  protected function getDirPath(FieldItemListInterface $itemList) {
    /** @var \Drupal\Core\Entity\ContentEntityInterface $parentEntity */
    $parentEntity = $itemList->getParent()->getValue();

    // Concanate path parts.
    $dir = implode('/', [
      $this->root,
      $this->themeHandler->getTheme($this->getSetting(static::FIELD_THEME))
        ->getPath(),
      $this->getSetting(static::FIELD_DIR),
      $parentEntity->getEntityTypeId(),
      $parentEntity->bundle(),
      $this->getSetting(static::FIELD_GROUP),
    ]);

    return is_dir($dir) ? $dir : FALSE;
  }

  /**
   * Return the list of thumbnails path.
   *
   * @param string $dirPath
   *   Tje dir path.
   *
   * @return string[]
   *   List of image paths.
   */
  protected function getImgPath(string $dirPath) {

    /** @var \Drupal\Core\File\FileSystemInterface $fileSystem */
    $imgPaths = $this->fileSystem->scanDirectory($dirPath, '*.*', ['recurse' => FALSE]);

    // Order imgPaths by the name of the pictures.
    $this->sortImgPath($imgPaths);

    return $imgPaths;
  }

  /**
   * Return the form element.
   *
   * @param array $element
   *   The element.
   * @param array $imgPaths
   *   The list of img path.
   * @param mixed|null $defaultValue
   *   The default value.
   *
   * @return array
   *   The form element.
   */
  protected function getFormElement(array $element, array $imgPaths, $defaultValue): array {
    // Attach libraries.
    $element['value']['#attached']['library'][] = 'view_mode_selector_field/view_mode_selector_field';

    // Build input.
    $element['#type'] = 'item';
    $element['#required'] = TRUE;
    $element['value']['#type'] = 'radios';
    $element['value']['#default_value'] = $defaultValue;
    $element['value']['#required'] = TRUE;
    $element['value']['#options'] = [];
    foreach ($imgPaths as $imgData) {
      $element['value']['#options'][$this->getSetting(static::FIELD_GROUP) . '/' . $imgData->name] =
        sprintf('<img src="%s" />',
          str_replace(\DRUPAL_ROOT, '', $imgData->uri)
        );
    }

    return $element;
  }

  /**
   * Order imgPaths by the name of pictures.
   *
   * @param array $imgPaths
   *   The list of img path.
   */
  protected function sortImgPath(array &$imgPaths) {
    uasort($imgPaths, function ($a, $b) {
      $a = basename($a->uri);
      $b = basename($b->uri);
      if ($a == $b) {
        return 0;
      }

      return ($a < $b) ? -1 : 1;
    });
  }

}
